
% identifica i blobs, quindi le strisce
[centers, centroids, boxes] = findStreaks(immasked, maxBlobs, blobsThreshold);
[m,n] = size(centroids);
speeds = zeros(1,n);
streaks = zeros(2,2,n);

% strutture di output
ocenters = zeros(2,n);
ocentroids = zeros(2,n);
oboxes = zeros(n,4);
ospeeds = zeros(1,n);
ostreaks = zeros(2,2,n);
i = 0;
lxaxis = [ 0 1; 0 0]; % linea asse x
for k=1:n
    box = boxes(k,:);
    centroid = centroids(:,k);
    center = centers(:,k);
    
    % definisci linea della direzione della striscia
    l1 = [centroid v1_c];
    
    % calcola l'inclinazione rispetto all'asse x
    theta = computeLinesAngle(l1, lxaxis, 1);
    
    % scegli l'immagine sorgente
    if(acfOn == 2)
        imsource = imd;
    else
        imsource = immasked;
    end
    
    % scegli se ritagliare la porzione pi piccola di immagine
    % che contiene la striscia considerata
    if(applyBoxing)
        [imresized, scaledcentroid] = getBoundingBoxImage(imsource, centroid, box, boxScalefactor);
    else
        imresized = imsource;
        scaledcentroid = centroid;
    end
    
    %if(acfOn == 2)
    %    [imresized, scaledcentroid] = getBoundingBoxImage(imd, centroid, box, boxScalefactor);
    %else
    %    [imresized, scaledcentroid] = getBoundingBoxImage(immasked, centroid, box, boxScalefactor);
    %end
    
    % ruota di theta gradi l'immagine ritagliata in modo tale che
    % la stricia sia orizzontale, quindi parallela all'asse x!
    [imR, centroidR] = rotate_image( rad2grad(theta), imresized, scaledcentroid );
    
    % ritaglia solo le linee interessanti a partire dall'ordinata del
    % centroide. Quindi da centroide-expand a centroide+expand
    y = int32(centroidR(2));
    delta = fix(streakHeight / 2);
    [lb, ub] = checkBounds(imR, (y - delta), (y + delta), 'height');
    imstreak = imR(lb:ub,:);
            
    % filtro derivata
    dx = [1 0 -1; 1 0 -1; 1 0 -1];
    imstreakdx = conv2(imstreak, dx, 'same');
    
    % elimina background nero che disturba la derivata
    if(acfOn == 2)
        imstreakdx = cropBackground(imstreak, imstreakdx);
    end
    
    if(showComputation)
        figure();
        
        % striscia considerata
        subplot(2,3,1);
        hold on;
        title('Streak considerata');
        imshow(immasked);
        scatter(centroid(1), centroid(2),'filled');
        
        % striscia ruotata
        subplot(2,3,4);
        hold on;
        title('Streak ruotata');
        imshow(imR);
        scatter(centroidR(1), centroidR(2),'filled');
        line([1 size(imR,2)], [ub ub], 'Color', 'g');
        line([1 size(imR,2)], [lb lb], 'Color', 'g');
        xlabel(sprintf('Rotazione: %3.0f',rad2grad(theta)))
    end
    
    % calcola gli estremi della striscia
    [p1 p2 ex acf] = computeExtremePoints(imstreakdx, center, v1_c, boxScalefactor, showComputation);
    streak = [p1 p2];
    
    % calcola velocit approssimativa
    speed = computeSpeed(streak, v1_h, v2_h, ref_c, reflen, shutter);
    
    % stampa linee individuate
    figure(figStreaks);
    figure(figStreaks);
    hold on;
    scatter(center(1), center(2), 'filled');
    plotLine([v1_c streak(:,2)], 'g', 2, '--');
    scatter(streak(1,1), streak(2,1), 'filled', 'g');
    scatter(streak(1,2), streak(2,2), 'filled', 'g');
    
    % stampa velocit
    plotText(streak(:,1), streak(:,2), horzcat('\leftarrow', sprintf('%3.2f Km/h', speed)), 'r', 12);
    
    % filtra per velocit
    if((speed >= minSpeed) && (speed <= maxSpeed))
        i = i +1;
        
        % riempi strutture
        ocentroids(:,i) = centroid;
        ocenters(:,i) = center;
        oboxes(i,:) = box;
        ostreaks(:,:,i) = streak;
        ospeeds(i) = speed;
    end
    
    % riempi strutture
    streaks(:,:,k) = streak;
    speeds(k) = speed;
end

% ridimensiona
ocentroids = ocentroids(:,1:i);
ocenters = ocenters(:,1:i);
oboxes = oboxes(1:i,:);
ostreaks = ostreaks(:,:,1:i);
ospeeds = ospeeds(1:i);
