function [ p1 p2 extent yacf ] = computeExtremePoints( imd, c, dirp, scalefactor, verbose )
%COMPUTEEXTREMEPOINTS Calcola i punti estremi di un blur extent
%   Parametri:
%       imd:            derivata dell'immagine
%       c:              vettore colonna del punto centrale della
%                       striscia (blur extent)
%       dirp:           vettore colonna punto che determina la direzione
%                       della striscia, indica quindi la direzione
%                       della linea c,dirp
%       scalefactor:    fattore di scala

% calcola la funzione di autocorrelazione
yacf = acf(imd);
[m,n] = size(yacf);

% calcola la lunghezza totale dell'extent
pixthres = fix(0.15 * (n / scalefactor)); % soglia minima di pixel da considerare validi per l'acf
if((pixthres <= 0)||(pixthres >= n))
    pixthres = 1;
end

minvals = zeros(m,1);
extents = zeros(m,1);
for k=1:m
    [minv, exte] = min(yacf(k,pixthres:end));
    minvals(k) = minv;
    extents(k) = exte + pixthres;
end

% calcola l'extend secondo varie modalit
extent1 = median(extents);

extent2 = mean(extents);
[mm,pos] = min(abs(extents - ones(size(extents,2),1).*extent2));
extent3 = extents(pos,1);

yacf4 = sum(yacf,1);
[minval4, extent4] = min(yacf4(pixthres:end));
extent4 = extent4 + pixthres;

yacf5 = median(yacf,1);
[minval5, extent5] = min(yacf5(pixthres:end));
extent5 = extent5 + pixthres;

% migliore
extent = extent1;

% visualizza risultati
if(verbose)
    
    % mediana
    subplot(2,3,2);
    hold on;
    title('Mediana extents');
    plot(yacf');
    thline = [min(min(yacf)):max(max(yacf))];
    plot(pixthres * ones(1,size(thline,2)),thline,'--r');
    scatter(extents,minvals,'filled','r');
    xlabel(sprintf('Extent mediano: %5.0f',extent1));
    
    % media
    subplot(2,3,3);
    hold on;
    title('Media extents ed extent con scarto minimo');
    plot(yacf');
    
    plot(pixthres * ones(1,size(thline,2)),thline,'--r');
    scatter(extents,minvals,'filled','r');
    xlabel(sprintf('Extent medio: %5.0f - Extent scarto min: %5.0f',extent2, extent3));
    
    % somma acf
    subplot(2,3,5);
    hold on;
    title('Somma delle acf di ogni linea di pixel');
    plot(yacf4');
    thline = [min(yacf4):max(yacf4)];
    plot(pixthres * ones(1,size(thline,2)),thline,'--r');
    scatter(extent4,minval4,'filled','r');
    xlabel(sprintf('Min somma punti: %5.0f',extent4));
    
    % mediana acf
    subplot(2,3,6);
    hold on;
    title('Mediana delle acf di ogni linea di pixel');
    plot(yacf5');
    thline = [min(yacf5):max(yacf5)];
    plot(pixthres * ones(1,size(thline,2)),thline,'--r');
    scatter(extent5,minval5,'filled','r');
    xlabel(sprintf('Min mediana punti: %5.0f',extent5));
end

% calcola i punti estremi appartententi alla linea c,dirp
% situati alle distanza in pixel pari a met extent
p1 = getPointAtDistance(c, dirp, extent/2);
p2 = getPointAtDistance(c, dirp, -extent/2);
