function [ len ] = computeLength( v1_h, v2_h, seg_c, ref_c, reflen )
%COMPUTELENGTH Calcola la lunghezza di un segmento parallelo ad uno di riferimento
%   Parametri:
%       v1_h:     vanishing point appartenente al piano (cord.omogenee)
%       v2_h:     vanishing point perperdicolare al primo appartenente al piano (cord.omogenee)
%       seg_c:    matrice di coppie di vettori colonna dei due punti del segmento da misurare(cord.cartesiane)
%       ref_c:    matrice di coppie di vettori colonna dei due punti del riferimento (cord.cartesiane)
%       reflen:   lunghezza reale del riferimento

% calcola vanishing line
l_h = cross(v1_h, v2_h);

% ordina punti
ref_c = sortrows(ref_c', -2)';
seg_c = sortrows(seg_c', -2)';

ref_h = cart2homo(ref_c);
seg_h = cart2homo(seg_c);

% calcola v3
v3_h = cross(cross(seg_h(:,1), seg_h(:,2)), cross(ref_h(:,1), ref_h(:,2)));

% calcola punto u sulla vanishing line
u_h = cross(cross(seg_h(:,1), ref_h(:,1)), l_h);

% calcola transfered point
tr_h = cross(cross(seg_h(:,2), u_h), cross(v3_h, ref_h(:,1)));

% calcola le distanze dalla base del riferimento
ref_c = homo2cart(ref_h);
tr_c = homo2cart(tr_h);
v3_c = homo2cart(v3_h);

dtr = norm(tr_c - ref_c(:,1));
dref = norm(ref_c(:,2) - ref_c(:,1));
dv3 = norm(v3_c - ref_c(:,1));

% lunghezza (cross-ratio)..
len = ((dtr .* (dv3 - dref))./(dref .* (dv3 - dtr))) .* reflen;
