function [ mask ] = road2mask(imd, v, points_c)
%ROAD2MASK Crea un maschera della strada
%   Parametri:
%       imd:        immagine in scala di grigi
%       v:          vanishing point longitudinale
%       point_c:    4 punti che delimitano la strada
%                   [x1 x2 x3 x4;y1 y2 y3 y4]

points_h = cart2homo(points_c);

[M,N,P] = size(imd);

extreme_c = [1 1 N N;1 M M 1];

extreme_h = cart2homo(extreme_c);

lextreme_h = zeros(3,4);
lextreme_h(:,1) = cross(extreme_h(:,1),extreme_h(:,2));
lextreme_h(:,2) = cross(extreme_h(:,2),extreme_h(:,3));
lextreme_h(:,3) = cross(extreme_h(:,3),extreme_h(:,4));
lextreme_h(:,4) = cross(extreme_h(:,4),extreme_h(:,1));

lstreet_h = zeros(3,2);
lstreet_h(:,1) = cross(points_h(:,1),points_h(:,2));
lstreet_h(:,2) = cross(points_h(:,3),points_h(:,4));

extreme = zeros(3,3,4);
extreme(:,:,1) = [lextreme_h(:,1) extreme_h(:,1) extreme_h(:,2)];
extreme(:,:,2) = [lextreme_h(:,2) extreme_h(:,2) extreme_h(:,3)];
extreme(:,:,3) = [lextreme_h(:,3) extreme_h(:,3) extreme_h(:,4)];
extreme(:,:,4) = [lextreme_h(:,4) extreme_h(:,4) extreme_h(:,1)];

street_h = zeros(3,3,2);
street_h(:,1,1) = lstreet_h(:,1);
street_h(:,1,2) = lstreet_h(:,2);

for k = 1:2
    index = 2;
    for j = 1:4
        p_h = cross(lstreet_h(:,k),extreme(:,1,j));
        p_c = homo2cart(p_h);    
        
        if(isPointInSegmentBounds(homo2cart(extreme(:,2,j)),homo2cart(extreme(:,3,j)),p_c, 1) == 1)
            street_h(:,index,k) = p_h;
            index = index +1;
        end
    end
end

street_c(:,:,1) = homo2cart(street_h(:,:,1));
street_c(:,:,2) = homo2cart(street_h(:,:,2));

%sort by y
for k = 1:2
    p = street_c(:,:,k)';
    p = sortrows(p,2);
    street_c(:,:,k) = p';
end

pointPolygon_c = [];
for j = 2:3
    if(j==2)
        alfa = street_c(:,j,1);
        beta = street_c(:,j,2);
    else
        alfa = street_c(:,j,2);
        beta = street_c(:,j,1);       
    end
    
    pointPolygon_c = [pointPolygon_c alfa]; 
    for t = 1:4
        if(isPointInSegmentBounds(alfa, beta, extreme_c(:,t),1) == 1)
            pointPolygon_c = [pointPolygon_c extreme_c(:,t)];
        end
    end
    pointPolygon_c = [pointPolygon_c beta];
end

pointPolygon2_c = [];
[m,n] = size(pointPolygon_c);
for k = 1:n
    if(externToVanishing(imd,v,pointPolygon_c(:,k)) == 0)
        pointPolygon2_c = [pointPolygon2_c pointPolygon_c(:,k)];
    else
        pointPolygon2_c = [pointPolygon2_c v];
    end
end

mask = poly2mask(pointPolygon2_c(1,:),pointPolygon2_c(2,:),M,N);
